/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 * 		 2003 OGINO Tomonori <ogino@nn.iij4u.or.jp>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* compressedfile.cpp */

#include "compressedfileitem.h"

#include "extract.h"
#include "directoryview.h"
#include "imageviewer.h"
#include "compressedimagefileiconitem.h"
#include "imagelistview.h"
#include "mainwindow.h"
#include "directoryview.h"
#include "directory.h"

#include <qlistview.h>
#include <qfileinfo.h>
#include <qapplication.h>

#include <kprocess.h>
#include <kiconloader.h>
#include <kio/job.h>
#include <klocale.h>

#include <qtextcodec.h>

CompressedFileItem::CompressedFileItem (Directory *parent, const char *filename, const char *path,
					MainWindow *mw,	DirectoryView* dirView,
					ImageViewer * iv, ImageListView * imageList)
	:ListItem(parent, dirView, iv, imageList, mw),
	 f (filename)
{
	readable = TRUE;
	full.append (path);
	full.append (filename);

	list.setAutoDelete (TRUE);
	QFileInfo * info = new QFileInfo (full);
	size = new QString ();
	size->sprintf ("%10d", info->size ());
	extension = new QString (info->extension());

	setPixmap(0, BarIcon(KMimeType::iconForURL(full), KIcon::SizeSmall ));
	setDropEnabled (FALSE);
}



QString
CompressedFileItem::text (int column) const
{
	if (column == 0)
	{
		return f.name ();
	}
	else
	if (column == 1)
		return (*extension);
	else
		return (*size);
}

void
CompressedFileItem::removeImage (CompressedImageFileIconItem * imf)
{
	if (list.findRef (imf))
	{
		list.removeRef (imf);
	}
}

void
CompressedFileItem::load (bool refresh)
{
	if(!imageList->hasImages()) iv->loadImage(NULL);
	QApplication::setOverrideCursor (waitCursor); // this might take time
	mw->setMessage(i18n("Loading '")+text(0)+"'...");

	Extract *extract = new Extract (full);
	QStringList::iterator s;
	CompressedImageFileIconItem *item;

	for (s = extract->files.begin(); s != extract->files.end(); ++s)
	{
		if(dirView->isImage(new QFileInfo(*s)))
		    {
			item = new CompressedImageFileIconItem (
						imageList,
						this,
						(const char*)text(0),
						(const char*)*s,
						(const char*)*s,
						iv);
			list.append (item);

			mw->slotAddImage();
		}
	}
	if(refresh)
	{
		imageList->slotLoadFirst();
	}

	mw->setMessage(i18n("Ready"));
	QApplication::restoreOverrideCursor ();       // restore original cursor
}

void
CompressedFileItem::unLoad()
{
	if(mw->preview())
		imageList->stopLoading();
	
	KShellProcess *proc = new KShellProcess ();

	QString dest;
	dest.setNum (getpid ());
	dest.prepend ("/tmp/.qlist");
	dest.append ("/");
	dest.append (KShellProcess::quote(text (0)));

	*proc << "rm -f -r";
	*proc << (const char *) dest;
	proc->start (KShellProcess::Block);

	mw->slotRemoveImage(list.count());
	list.clear ();
	iv->updateStatus ();
	imageList->slotUpdate();
	if(mw->preview())
		imageList->slotLoadFirst();
}

void
CompressedFileItem::rename(QString )
{
	qWarning("%s %d TODO CompressedFileItem::rename(QString newDirName)",__FILE__,__LINE__);
}
void
CompressedFileItem::rename()
{
	qWarning("%s %d TODO CompressedFileItem::rename()",__FILE__,__LINE__);
}
void
CompressedFileItem::properties()
{
	qWarning("%s %d TODO CompressedFileItem::properties()",__FILE__,__LINE__);
}
void
CompressedFileItem::create(QString )
{
	qWarning("%s %d TODO CompressedFileItem::create(QString dirName)",__FILE__,__LINE__);
}

bool
CompressedFileItem::refresh(bool )
{
	qWarning("%s %d TODO CompressedFileItem::refresh()",__FILE__,__LINE__);
	return true;
}
