/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* directory.h */

#ifndef __DIRECTORY_H__
#define __DIRECTORY_H__

#include "listitem.h"

#include <qfile.h>
#include <qptrlist.h>

class ImageListView;
class ImageFileIconItem;
class DirFileIconItem;
class FileIconItem;
class ImageViewer;
class MainWindow;
class DirectoryView;

class QFile;

class KAction;

class Directory:public ListItem
{
public:
  Directory (DirectoryView* parent, ImageViewer * iv, ImageListView * imageList, MainWindow * mw);
  Directory (Directory * parent, const char *filename, DirectoryView* dirView, ImageViewer * iv,  ImageListView * imageList, MainWindow * mw);
  ~Directory();

  QString text (int column) const;

  QString name ();
  QString path ();

  void setOpen (bool);
  void recursivelyOpen();

  void setup ();

  void load (bool refresh=true);
  void unLoad ();
  bool refresh(bool preview=true);
  
  virtual ListItem *find (const char *);
  void goTo (const char *dest);
  void loadFirst ();

  void removeImage (ImageFileIconItem * imf);

  void createDir(QString dirName);
  void createAlbum(QString albumName);
  void rename(QString newDirName);
  void rename();
  void properties();

protected :
	void init();
	int size;

private:
  QFile f;
  Directory *p;
  bool readable;
  QString newDirName;
  QPtrList < FileIconItem > list;
};


#endif
