/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "fileiconitem.h"


#include "imageviewer.h"
#include "imagelistview.h"

#include <kurl.h>
#include <kiconloader.h>
#include <kfileitem.h>
#include <kfileiconview.h>

#include <qapplication.h>
#include <qiconview.h>
#include <qstring.h>

FileIconItem::FileIconItem(ImageListView *parent, const char* path, const char* filename, const char* , ImageViewer *iv)
	: KFileIconViewItem (parent,
			filename,
			NULL,
			new KFileItem(KFileItem::Unknown,
					KFileItem::Unknown,
					KURL(QString("file:")+path+filename),
					false))

{
	this->imageList=parent;
	this->iv=iv;
	haspreview=false;
	__isimage__=false;
	__ismovable__=false;

	size=NULL;extension=NULL;type=NULL;date=NULL;description=NULL;info=NULL;

}

FileIconItem::~FileIconItem()
{
	delete(size);delete(extension);delete(type);
	delete(date);delete(description);delete(info);
}

void
FileIconItem::setName(QString n){
	myName=n;
};

void
FileIconItem::setType(QString type){
	this->type = new QString(type);
};


void
FileIconItem::setPixmap ( const QPixmap& icon, bool haspreview )
{
	KFileIconViewItem::setPixmap(icon);
	this->haspreview=haspreview;
}
void 
FileIconItem::setHasPreview (bool preview)
{
	this->haspreview=preview;
}

bool
FileIconItem::hasPreview ()
{
	if(!haspreview) 
		return false;
	else
	{
		bool older=QFileInfo(fullName()).lastModified() < QFileInfo (QDir::homeDirPath () + "/.showimg/cache/" + fullName()).lastModified();
		return older;
	}

}

FileIconItem*
FileIconItem::nextItem ()
{
	return (FileIconItem*)KFileIconViewItem::nextItem();
}

FileIconItem*
FileIconItem::prevItem ()
{
	return (FileIconItem*)KFileIconViewItem::prevItem();
}


QString
FileIconItem::name()
{
	return myName;
};

QString 
FileIconItem::fullName()
{
	return "";
};

bool
FileIconItem::suppression(bool)
{
	return false;
};

bool
FileIconItem::suppression()
{
	return false;
};

bool
FileIconItem::moveToTrash()
{
	return false;
};

bool
FileIconItem::shred()
{
	return false;
}

QString
FileIconItem::text(int ) const
{
	return QString();
}

void
FileIconItem::setWallpaper()
{
}

QString
FileIconItem::getFileName(QString )
{
	return QString();
};

QString
FileIconItem::getFileExt(QString )
{
	return QString();
};

QString
FileIconItem::getFullName(QString )
{
	return QString();
};

QString
FileIconItem::getFullPath(QString )
{
	return QString();
};


QString
FileIconItem::getFileName(QString *fullName)
{
	int debut = fullName->findRev ("/");
	int fin = fullName->findRev (".");
	return fullName->mid(debut+1, fin-debut-1);
}


QString
FileIconItem::getFileExt(QString *fullName)
{
	return  fullName->right (fullName->length () - fullName->findRev (".") - 1);
}

QString
FileIconItem::getFullName(QString *fullName)
{
	return fullName->right (fullName->length () - fullName->findRev ("/") - 1);
}

QString
FileIconItem::getFullPath(QString *fullName)
{
	return fullName->left(fullName->findRev ("/") + 1);
}

QString
FileIconItem::mimetype() const
{
	return fileInfo()->mimetype();
}

void
FileIconItem::setKey ( const QString & k )
{
	if(k=="name")
		KIconViewItem::setKey(text());
	else
	if(k=="size")
		KIconViewItem::setKey(*size);
	else
	if(k=="type")
		KIconViewItem::setKey(*extension);
	else
	if(k=="date")
		KIconViewItem::setKey(*date);
	else
		KIconViewItem::setKey(text());

}

void
FileIconItem::setSelected (bool s)
{
	KIconViewItem::setSelected (s);
	qApp->processEvents();
	if (s)
	{
		QApplication::setOverrideCursor (waitCursor);	  // this might take time
		iv->loadImage ((const char *)(fullName ()), index());
		qApp->processEvents();
		if (imageList->preloadIm() && nextItem())
			iv-> preloadImage ((const char *)(((FileIconItem *) (nextItem ()))-> fullName ()));
		QApplication::restoreOverrideCursor ();   // restore original cursor
	}
	else
	{
	}
}

QString
FileIconItem::toolTipStr() const
{
	return "TODO";
}



bool
FileIconItem::isImage() const
{
	return __isimage__;
}

bool
FileIconItem::isMovable() const
{
	return __ismovable__;
}






