/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* imageliste.h */

#ifndef _IMAGELISTVIEW_H_
#define _IMAGELISTVIEW_H_

#include <kfileiconview.h>

#include <qtooltip.h>

#include "describe.h"


//#include "fileiconitem.h"

class FileIconItem;
class MainWindow;
class ImageLoader;
class ImageListView;

class KProcess;
class KPopupMenu;
class KFileViewItem;
class KAction;
class KIconEffect;

class QListViewItem;
class QIconViewItem;

class KToolTip : public QToolTip
{
public:
	KToolTip(QWidget *parent, ImageListView *imageList);
	virtual ~KToolTip(){};

	void maybeTip(const QPoint &pos);
	void tip ( const QRect & r, const QString & text );

private:
	ImageListView *imageList;
};



class ImageListView:public KIconView
{
	Q_OBJECT;
public:
	ImageListView (QWidget * parent, const char *name, MainWindow * mw);

	void initMenu(KActionCollection *actionCollection);

	void mousePress (QMouseEvent * e);

	void contentsMouseReleaseEvent ( QMouseEvent * e );
	void contentsMouseDoubleClickEvent ( QMouseEvent * e );
	void contentsMouseMoveEvent ( QMouseEvent *e );
	void mouseMoveEvent(QMouseEvent * e );

	virtual void setItemTextPos ( ItemTextPos pos );

	bool doPreload();
	void setPreload(bool p);

	bool doLoop();
	void setLoop(bool loop);

	bool doRandom();
	void setRandom(bool ran);

	bool checkTrash();
	void setCheckTrash(bool trash);
	bool checkDelete();
	void setCheckDelete(bool del);
	bool checkShred();
	void setCheckShred(bool shred);

	/**
		return true if it have to show ALL files
	*/
	void setPreloadIm(bool prel);
	bool preloadIm();


	FileIconItem* firstItem ();
	FileIconItem* currentItem();
	FileIconItem* lastItem();
	FileIconItem* findItem (const QString & text, bool fullname=false);
	FileIconItem* itemAt(QPoint pos);

	void refresh ();
	void stopLoading ();
	void setThumbnailSize(QSize newSize);

	void sort();

	QString currentItemName();
	void setCurrentItemName(QString itemName, bool select=true);
	bool hasImages();
	bool hasImageSelected();

	bool inFullScreen;
	bool preview;
	ImageLoader * il;
	int currentIconSize;


signals:
	void loadFinish();

protected:
	int sortMode;
	
	QDragObject* dragObject();
	void showEvent ( QShowEvent * );
	void leaveEvent ( QEvent * ) ;

protected slots:
	void selectionChanged();
	void deletionDone( KIO::Job *);
	void highlight(QIconViewItem *item);
	void onViewport();
	void slotDescribeClose();

public slots:
	void slotByName();
	void slotByExtension();
	void slotBySize();
	void slotByDate();

	void slotWallpaper ();
	void slotGimp ();
	void slotEndGimp (KProcess *proc);

	void slotSupprimmer ();
	void slotMoveToTrash();
	void slotShred();

	void slotFilesMoveTo();
	void slotFilesCopyTo();

	void slotFileProperty();
	void slotImageProperty();
	void slotImageInfo();

	void next ();
	void previous ();
	void first ();
	void last ();

	void slotOpenWith();
	void slotRename();

	void slotSetPixmap (const QPixmap pm);

	void slotLoadFirst ();
	void slotLoadFirst(FileIconItem *item);
	void slotLoadNext ();
	void slotResetThumbnail();

	void slotInvertSelection();
	void slotUnselectAll();
	void slotSelectAll();
	virtual void slotUpdate ();

private:
	bool trash, del, shred;
	QPixmap *currentIconItem;
	QIconViewItem *curIt;
	KIconEffect *iconEffect;

	Describe *dscr;

	int nbrTh;

	bool loop; //loop icons
	bool p; //preload
	bool random; //random
	bool __preload__;

	MainWindow *mw;
	FileIconItem *imageLoading;

	KPopupMenu *popup;
	KActionCollection *actionCollection;

	KToolTip *toolTips;
};





#endif
