/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "imageloader.h"
#include "imagelistview.h"
#include "imagefileinfo.h"
#include "exif.h"

#include <kurl.h>
#include <kapp.h>
#include <klocale.h>
#include <kprocess.h>
#include <kpixmapio.h>
#include <kapplication.h>

#include <qstring.h>
#include <qimage.h>
#include <qlabel.h>
#include <qdir.h>
#include <qpainter.h>

//#include <iostream.h>
#include <sys/stat.h>
#include <unistd.h>
#include <math.h>

/* XPM */
static const char *bgxpm[] = {
"16 16 3 1",
"       c None",
".      c #656765",
"+      c #9B999B",
"++++++++........",
"++++++++........",
"++++++++........",
"++++++++........",
"++++++++........",
"++++++++........",
"++++++++........",
"++++++++........",
"........++++++++",
"........++++++++",
"........++++++++",
"........++++++++",
"........++++++++",
"........++++++++",
"........++++++++",
"........++++++++"};


/* XPM */
static const char * border_xpm[] = {
"154 134 54 1",
" 	c None",
".	c #FFFFFF",
"+	c #FBFBFB",
"@	c #F7F7F7",
"#	c #F3F3F3",
"$	c #EFEFEF",
"%	c #E9E9E9",
"&	c #E4E4E4",
"*	c #E1E1E1",
"=	c #DEDEDE",
"-	c #D9D9D9",
";	c #CCCCCC",
">	c #D5D5D5",
",	c #C2C2C2",
"'	c #CECECE",
")	c #BCBCBC",
"!	c #ADADAD",
"~	c #A0A0A0",
"{	c #B7B7B7",
"]	c #969696",
"^	c #C4C4C4",
"/	c #F8F8F8",
"(	c #F0F0F0",
"_	c #8E8E8E",
":	c #A6A6A6",
"<	c #BEBEBE",
"[	c #E6E6E6",
"}	c #F1F1F1",
"|	c #F9F9F9",
"1	c #898989",
"2	c #A2A2A2",
"3	c #B8B8B8",
"4	c #D1D1D1",
"5	c #868686",
"6	c #9D9D9D",
"7	c #CFCFCF",
"8	c #E2E2E2",
"9	c #838383",
"0	c #9B9B9B",
"a	c #B5B5B5",
"b	c #CDCDCD",
"c	c #848484",
"d	c #9F9F9F",
"e	c #D3D3D3",
"f	c #A1A1A1",
"g	c #BABABA",
"h	c #ACACAC",
"i	c #D4D4D4",
"j	c #EBEBEB",
"k	c #D8D8D8",
"l	c #959595",
"m	c #939393",
"n	c #CACACA",
"o	c #ABABAB",
"..........................................................................................................................................................",
"..........................................................................................................................................................",
"..................+..+..++.++..++...++.++..++...++.++..++...++.++..++...++.++..++...++.++..++...++.++..++...++.++..++...++.++..++.........................",
"...........+++@+@+@@+@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@++++...........",
".........++@@###$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$###@@++.........",
"........+@#$$%&&**************************************************************************************************************************&&%$#@@++.......",
"......++@$#................................................................................................................................=-=*%#@++......",
".....++##....................................................................................................................................;>=&$@+......",
".....+@#......................................................................................................................................,'-&$@+.....",
"....+@$........................................................................................................................................)'-&#@.....",
"....+##........................................................................................................................................))'=%#+....",
"...+@$..........................................................................................................................................!,>&#@+...",
"...+#%..........................................................................................................................................~{'=$@+...",
"...+#%..........................................................................................................................................]!^-&#+...",
".../(&........                                                                                                                          ........_:<>[}|...",
".../(&........                                                                                                                          ........1234[}@...",
"../@$*........                                                                                                                          ........56378$@...",
"...@$*........                                                                                                                          ........56{78$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90a78$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90a78$@...",
"../@$*........                                                                                                                          ........90ab>$@...",
"...@$*........                                                                                                                          ........90ab8}@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{b>}@...",
"../@$*........                                                                                                                          ........90{b8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"../@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab>}@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90{b8$@...",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{b8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab>}@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90{b8$@...",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90a78$@...",
"...@$*........                                                                                                                          ........90a78$@|..",
"...@$*........                                                                                                                          ........90ab>$@...",
"../@$*........                                                                                                                          ........90ab8}@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"../@$*........                                                                                                                          ........90ab>$@...",
"...@$*........                                                                                                                          ........90ab8}@...",
"...@$*........                                                                                                                          ........90{b8$@|..",
"...@$*........                                                                                                                          ........90{b8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{b>}@...",
"../@$*........                                                                                                                          ........90{b8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"../@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab>}@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90{b8$@...",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{b8$@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90ab>}@...",
"../@$*........                                                                                                                          ........90ab8$@|..",
"../@$*........                                                                                                                          ........90{b8$@...",
"...@$*........                                                                                                                          ........90ab8$@...",
"../@$*........                                                                                                                          ........90ab8$@...",
"...@$*........                                                                                                                          ........90ab8$@|..",
"...@$*........                                                                                                                          ........90a78$@...",
"...@$*........                                                                                                                          ........90a78$@|..",
"...@$*........                                                                                                                          ........90ab>$@...",
"../@$*........                                                                                                                          ........90ab8}@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"...@$*........                                                                                                                          ........90{78$@...",
"...@$*........                                                                                                                          ........90{78$@|..",
"../@$*........                                                                                                                          ........90{78$@...",
"..+@$8..........................................................................................................................................cdae8$@...",
"...@$8..........................................................................................................................................cdab8$@...",
"...@(&..........................................................................................................................................1fge&$@...",
"..++(&&........................................................................................................................................h_:<i&(++..",
"...+(jk........................................................................................................................................9lh^kj(+...",
"...+@j=i......................................................................................................................................m1dan=j@+...",
"...+@$&eb....................................................................................................................................mclo<e&$@....",
"....+(j=bgn................................................................................................................................h91logb=j(++...",
"....+@$&kngodl_1cccccc99999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999cc9ccc1_ldognk&(@+....",
".....+@$&kb<ah:fdddddd000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000dddddf:ha<bk&$@+.....",
".....++($&=en^<ggaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaagg<^ne=&$@+......",
".......+@(j&=kiebbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbeik=&j$@++......",
".......++@($jj&&88888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888&&&$$(@++.......",
"..........+@@(((($$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$((((@@++.........",
"...........++++@@@++@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@++@@@@++++............",
"..................+...++.++.+.+.+.++.++.+.+.+.+..+.+.+.+.+.+.+.++.+.+.++.++.+.+.+.++.++.+.+.+.+..+.+.+.+.+.+.+.++.+.+.++.++.+.+..+.++.+...................",
"..........................................................................................................................................................",
".........................................................................................................................................................."};

class ImageListView;

void *
__thread_start (void *arg)
{
	pthread_cleanup_push (__thread_cleanup, arg);
	((ImageLoader *) arg)->thread_start ();
	pthread_cleanup_pop (0);
	pthread_detach (pthread_self ());
	return 0;
}

void
__thread_cleanup (void *arg)
{
	((ImageLoader *) arg)->thread_cleanup ();
}


ImageLoader::ImageLoader (QWidget * parent, const char *name):
QObject (parent, name)
{
	EventList.setAutoDelete (true);
	installEventFilter (this);
	Loading = Running = false;

	size=QSize(80,60);
	setThumbnailSize(size);
	showFrame=false;
	
	ptop=QImage(border_xpm);
}

ImageLoader::~ImageLoader ()
{
	stopLoading (true);
}

void
ImageLoader::setShowFrame(bool show)
{
	showFrame=show;
}
void
ImageLoader::setUseEXIF(bool useexif)
{
	useEXIF=useexif;
}


QPixmap
ImageLoader::addBorder(QPixmap *pix, bool hasAlpha)
{
	if(!showFrame) return *pix;

	QPixmap res(pix->size());
	QPainter p(&res);
		if(hasAlpha) p.drawTiledPixmap (0, 0, pix->width(), pix->height(), QPixmap(bgxpm));
		p.drawImage(0, 0, ptop.scale(pix->width()+1, pix->height()+1));
		p.drawImage((int)floor((float)pix->width()/ptop.width()*14), (int)floor((float)pix->height()/ptop.height()*13),
				pix->convertToImage().smoothScale( (int)ceil(pix->width()*0.79738562092)+1, (int)ceil(pix->height()*0.76691729323)+1));
	p.end();
	//delete(pix);
	return res;
}


void
ImageLoader::setThumbnailSize(QSize newSize)
{
	if(size==newSize)
		return;
	else
		size=newSize;
}


QSize
ImageLoader::getThumbnailSize()
{
	return size;
}


void
ImageLoader::loadMiniImage (QFileInfo * fi, QWidget * w, bool threaded)
{
	bool cont=true;
	QFileInfo thumb = QFileInfo (QDir::homeDirPath () + "/.showimg/cache/" + fi->absFilePath ());

	if(useEXIF && thumb.extension().lower()=="jpg" && !thumb.exists()  )
	{
		ProcessFile(fi->absFilePath(), false, thumb.absFilePath());
	}

	if (thumb.exists ())
	{
		if(fi->lastModified() < thumb.lastModified())
		{
			QImage im (QDir::homeDirPath () + "/.showimg/cache/" + fi->absFilePath (), "JPEG");
			double
			 wexpand = (double) im.width () / (double)getThumbnailSize().width(),
			 hexpand = (double) im.height () / (double)getThumbnailSize().height();
			if (wexpand != 1.0 && hexpand != 1.0) // don't expand small images  !
			{
	 			int neww, newh;
	 			float     s;

	 			if (wexpand > hexpand)
	 			{
	 	  			neww = (int) (im.width () / wexpand );
	 	  			newh = (int) (im.height () / wexpand );
	 	  			s = 1 / wexpand;
	 			}
	 			else
	 	 		{
	 	 			neww = (int) (im.width () / hexpand );
	 	 			newh = (int) (im.height () / hexpand );
	 	 			s = 1 / hexpand;
	 			 }
				im=im.scale(neww,newh);
			}
			cont=false;
			QPixmap pix = KPixmapIO().convertToPixmap(im);
			((ImageListView *) w)->slotSetPixmap(addBorder(&pix, im.hasAlphaBuffer()));
		}
	}

	if(cont)
	{
		ImageLoadEvent *e = new ImageLoadEvent (fi, w, threaded);
		EventList.append (e);
		if (EventList.count () > 0 && !Running)
		{
			Running = true;
			startTimer (1);
			nextImage ();
		}
	}
}

void
ImageLoader::startLoading ()
{
	Running = true;
	ImageLoadEvent *e = ((int) (EventList.count ()) > 0 ? EventList.take (0) : 0);
	if (!e)
	{
		Running = Loading = false;
		killTimers ();
	
		return;      
	}
	if (!initLoading (e))
	{
		cantLoad (e);
		return;
	}
	Loading = true;
	loadImageInternal (e);
}


void
ImageLoader::stopLoading (bool clean)
{
	if (Loading)
	{
		pthread_cancel (ThreadID);
		pthread_join (ThreadID, NULL);
		Loading = Running = false;
		killTimers ();
		ImageLoadedList.clear ();
	}

	if (clean)
	{
		EventList.clear ();
	}
}

void
ImageLoader::cantLoad (ImageLoadEvent * e)
{
	(void)kapp->postEvent (e->widget (), e);
	// Try to load next image
	Loading = false;
	nextImage ();
}

bool
ImageLoader::eventFilter (QObject *, QEvent * e)
{
	switch (e->type ())
	{
		case Event_NextImage:
			startLoading ();
			return true;
		case Event_ImageLoad:
		{
		 	 Loading = false;
		 	 ImageLoadEvent *ev = new ImageLoadEvent (*((ImageLoadEvent *) e));
		 	 finishLoading (ev);
		 	 (void)kapp->postEvent (ev->widget (), ev);
		 	 nextImage ();
		 	 return true;
		}
		default : return false;
	}
	return false;
}


void
ImageLoader::nextImage ()
{
	if (!Loading)
	{
		NextImageEvent *e = new NextImageEvent;
		(void)kapp->postEvent (this, e);
	}
}


void
ImageLoader::thread_start ()
{
	InternalImage.load ((const char*)InternalPath);
	ImageLoadedList.append (InternalEvent);
}


void
ImageLoader::thread_cleanup ()
{
}


void
ImageLoader::timerEvent (QTimerEvent * )
{
	while (ImageLoadedList.count () > 0)
	{
		ImageLoadEvent *e = ImageLoadedList.take (0);
		(void)kapp->postEvent (this, e);
	}
}


bool
ImageLoader::initLoading (ImageLoadEvent * e)
{ 
	QFileInfo *fi = e->fileInfo ();
	image_path = QString (fi->absFilePath ());
	image_url = KURL (image_path);

	if (!mini_image_file_exists || mini_image_outdated)
		return true;
	return false;
}


void
ImageLoader::finishLoading (ImageLoadEvent * e)
{
	QFileInfo *fi = e->fileInfo ();
	QImage image = InternalImage, ditheredImage;

	if (image.isNull ())
	{
		image = BarIcon("file_broken",48).convertToImage();
	}	
	
	double
	  wexpand = (double) image.width () / (double)getThumbnailSize().width() ,
	  hexpand = (double) image.height () / (double)getThumbnailSize().height();

	if (wexpand >= 1.0 || hexpand >= 1.0) // don't expand small images  !
	{
		int neww, newh;
		float	  s;
		if (wexpand > hexpand)
		{
		 	 neww = (int) (image.width () / wexpand);
		 	 newh = (int) (image.height () / wexpand);
		 	 s = 1 / wexpand;
		}
		else
		{
			neww = (int) (image.width () / hexpand);
			newh = (int) (image.height () / hexpand);
			s = 1 / hexpand;
		}
		image = image.smoothScale (neww, newh);
		
		QCString  dest = QCString (QDir::homeDirPath () + "/.showimg/");
		QDir	  dirDest = QDir (dest);
		if (!dirDest.exists ())
		{
			QDir ().mkdir (dest);
		}
		dest = dest + "cache/";
		dirDest = QDir (dest);
		if (!dirDest.exists ())
		{
			QDir ().mkdir (dest);
		}
		QCString  res = QCString (fi->absFilePath ());
		QCString  ssrep;
		int  pos = res.find ("/");
		while (pos != -1)
		{
			dest = dest + res.left (pos) + "/";
			dirDest = QDir (dest);
			if (!dirDest.exists ())
			{
				QDir ().mkdir (dest);
			}
			res.setStr (res.right (res.length () - pos - 1));
			pos = res.find ("/");
		}		
		image.save (QString (dest) + fi->fileName (), "JPEG");
	}		
	mini_image = KPixmapIO().convertToPixmap(image);	
	if(!mini_image.isNull ())
	{
		((ImageListView *) e->widget ())->slotSetPixmap (addBorder(&mini_image, image.hasAlphaBuffer()));
	}
	else
		((ImageListView *) e->widget ())->slotSetPixmap (BarIcon("file_broken",48));
	InternalImage.reset ();
	image.reset();
}


void
ImageLoader::loadImageInternal (ImageLoadEvent * e)
{
	InternalPath = image_url.path ();
	InternalEvent = e;
	InternalImage.reset ();
	// Problem here with xpm and xbm image, interfere with X system and can cause crash.
	// As it is probably a small image, it's probably not a problem to load it synchronously.

	if (!e->threaded ())
		thread_start ();
	else
		if (pthread_create (&ThreadID, 0, __thread_start, this) == 0);
	else
		qWarning ("%s %d  ImageLoader::loadImageInternal (ImageLoadEvent * e) : unable to start loading thread",__FILE__,__LINE__);

}
