/*
 * Copyright (C) 2000 Richard Groult <rgroult@jalix.org>
 * 		 2003 OGINO Tomonori <ogino@nn.iij4u.or.jp>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "main.h"
#include "kstartuplogo.h"

#include <kimageio.h>
#include <qpixmapcache.h>
#include <kprocess.h>

#include <dcopclient.h>

class KConfig;

extern void qimageio_readXCF(QImageIO *io);

int
main (int argc, char **argv)
{
	QString txt = i18n("(Please send me e-mail for bugs report...\nor encouragements :)");
	KAboutData*  aboutData = new KAboutData("showimg", I18N_NOOP ("showimg"),
	        	      "0.8", description, KAboutData::License_GPL,
	        	      "(c) 2003, Richard Groult",txt,
	        	      "http://www.jalix.org/projects/showimg/",
	        	      "rgroult@jalix.org" );

	aboutData->addAuthor ("Richard Groult", I18N_NOOP("Developer"), "rgroult@jalix.org", "http://ric.jalix.org");
	aboutData->addAuthor ("Dominik Seichter", I18N_NOOP("to allow me to use his great software krename for showimg, and patch writing"), "domseichter@web.de", "http://www.freshmeat.net/projects/krename" );
	aboutData->addAuthor ("Tomonori Ogino", I18N_NOOP("patch for archives"), "ogino@nn.iij4u.or.jp");

	aboutData->addCredit ("Valerie", I18N_NOOP("for the logo"));
	aboutData->addCredit ("Benoist", I18N_NOOP("BETA testor, translation, and help for the zoom feature"), "b.gaston@laposte.net");
	aboutData->addCredit ("Romain Vallet", I18N_NOOP("Documentation and spelling corrector"), "rom@jalix.org");

	aboutData->addCredit ("Alain Bidaud and the Jalix team", I18N_NOOP("for the web site"));

	aboutData->addCredit ("Andre Pascual ", I18N_NOOP("for the new logo"), "andre@linuxgraphic.org");
	aboutData->addCredit ("Jean-Philippe Martin",I18N_NOOP("for icons and advices about features and design") , "jeanphilippemartin@club-internet.fr");
	
	aboutData->addCredit ("Hape Schaal ", I18N_NOOP("German translator and great help for debug"), "hp.news@gmx.de");
	aboutData->addCredit ("Andrea Tavazzani",I18N_NOOP("Italian translation") , "andreatavazzani@libero.it");
	aboutData->addCredit ("E. Jorge Tizado ",I18N_NOOP("Spanish translation") , "dbajtm@unileon.es");
	aboutData->addCredit ("Fabrice Mous",I18N_NOOP("Dutch translation") , "fabrice@kde.nl");
	aboutData->addCredit ("Karolina Lindqvist",I18N_NOOP("Swedish translation") , "pgd-karolinali@algonet.se");
	aboutData->addCredit ("Guillaume Duhamel",I18N_NOOP("Beginning of Japanese translation") , "Guillaume.Duhamel@univ-rouen.fr");
	aboutData->addCredit ("Tomonori Ogino",I18N_NOOP("Japanese translation and archive patch") , "ogino@nn.iij4u.or.jp");
	aboutData->addCredit ("Jozef Riha",I18N_NOOP("Czech and Slovak translations") , "zefo@seznam.cz");

	aboutData->addCredit ("Matthias Wandel", I18N_NOOP("to allow me to use jhead for jpeg-exif format support"), "MWandel@rim.net");
	aboutData->addCredit ("Andrew Richards", I18N_NOOP("to allow me to use his printImageDialog class"), "ajr@users.sourceforge.net");

	aboutData->addCredit ("Lot's-of people", I18N_NOOP("for bugs reports, advices,\nand encouragements :)"));

	//
	KApplication::setColorSpec(KApplication::NormalColor);

	KCmdLineArgs::init (argc, argv, aboutData);
	KCmdLineArgs::addCmdLineOptions (options);
	KApplication kApp;

	KConfig *config = new KConfig("showimgrc");
	config->setGroup("confirm");
	KStartupLogo* logo=NULL;
	if(config->readBoolEntry("showSP", true))
	{
		logo = new KStartupLogo();
		logo->show();
	}

  KImageIO::registerFormats();
  QImageIO::defineIOHandler("XCF", "gimp xcf", 0, qimageio_readXCF, NULL);
  QPixmap::setDefaultOptimization(QPixmap::MemoryOptim);
  QPixmapCache::setCacheLimit(2*1024);

  MainWindow *frame=NULL;
  if (kApp.isRestored())
  {
    RESTORE(MainWindow);
  }
  else
  {
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs ();
    if (argc == 1)
    {
        frame = new MainWindow ("MainFrame");
    }
    else
    {
        frame = new MainWindow (QFile::decodeName (args->arg (0)), "MainFrame");
    }
    frame->setIcon(kApp.icon());
    kApp.setMainWidget (frame);
  }
  QObject::connect(qApp,SIGNAL(lastWindowClosed()),frame,SLOT(quit()));
  
	if(logo)
	{
		logo->hide();
		delete logo;
	}
  int ret = kApp.exec ();
  {
	/* clean-up temporaly extracting dir. */
	KShellProcess *proc = new KShellProcess();

	QString  dest;
	dest.setNum (getpid ());
	dest.prepend ("/tmp/.qlist");

	*proc << "rm -r -f " << KShellProcess::quote(dest);
	proc->start(KShellProcess::Block);
  }
  return ret;
}
